(in-package :ucw)

(defcomponent user-login (simple-window-component
                          form-element
                          status-bar-mixin)
  ((username :accessor username
             :initarg :username
             :component (text-element
                         :label "Username"
                         :size 10
                         :constraints '(:not-null))
             :documentation "User login name.")
   (password :accessor password
             :initarg :password
             :component (password-element
                         :label "Password"
                         :size 10
                         :constraints '(:not-null))
             :documentation "User password."))
  (:documentation "User login component.")
  (:default-initargs
    :label ""
    :title "User Login"
    :stylesheet "stylesheet.css"))

(defmethod render ((self user-login))
  (render (status-bar self))
  (call-next-method))

(defmethod render :before ((self user-login))
  (show-message "Please enter login and password."))

(defmethod report-error ((self user-login) slot-name condition)
  (show-message (format nil "~a - ~a~%"
                            (label (slot-value self slot-name)) condition)
                    :severity :error))

(defmethod render-slots :after (view (self user-login) slots)
  (declare (ignore view slots))
  (<:tr
    (<:td :colspan 2
          (<ucw:input :type "submit" :action (submit self) :value "Ok")
          (<ucw:input :type "submit" :action (cancel self) :value "Cancel"))))

(defaction cancel ((self user-login))
  (answer nil))

(defaction submit ((self user-login))
  (when (validp self)
      (aif (check-credentials self)
           (answer it)
           (show-message "Bad login or password." :severity :error))))

(defmethod check-credentials ((self user-login))
  (let* ((lv (lisp-value self))
         (username (cdr (assoc 'username lv)))
         (password (cdr (assoc 'password lv)))
         (user (find-application-user username)))
    (when (and user
               (check-user-password user password))
      user)))